@@ -50,23 +50,39 @@ fi
 # so that in particular what is mounted at '/' is output before other stuff.
 read_filesystems_command="$read_filesystems_command | sort -t ' ' -k 1,1 -u"
 
-# docker daemon mounts file systems for its docker containers
-# see also https://docs.docker.com/storage/storagedriver/device-mapper-driver/#configure-direct-lvm-mode-for-production
-# As it is for container usage only we do not to backup these up or recreate as this disk device is completely under
-# control by docker itself (even the recreation of it incl, the creation of the volume group). Usually this is
-# done via a kind of cookbook (Chef, puppet or ansible)
+# The Docker daemon mounts file systems for its Docker containers, see also
+# https://docs.docker.com/storage/storagedriver/device-mapper-driver/#configure-direct-lvm-mode-for-production
+# As it is for container usage only we do not to backup these up or recreate as this disk device is completely
+# under control by Docker itself (even the recreation of it incl, the creation of the volume group).
+# Usually this is done via a kind of cookbook (Chef, puppet or ansible).
 docker_is_running=""
-service docker status >/dev/null 2>&1 && docker_is_running="yes"
+docker_root_dir=""
+if service docker status &>/dev/null ; then
+    docker_is_running="yes"
+    # When the Docker daemon/service is running, try to get its 'Docker Root Dir':
+    # Kill 'docker info' with SIGTERM after 5 seconds and with SIGKILL after additional 2 seconds
+    # because there are too many crippled Docker installations, cf. https://github.com/rear/rear/pull/2021
+    docker_root_dir=$( timeout -k 2s 5s docker info | grep 'Docker Root Dir' | awk '{print $4}' )
+    # Things may go wrong in the 'Docker specific exclude part' below
+    # when Docker is used but its 'Docker Root Dir' cannot be determined
+    # cf. https://github.com/rear/rear/issues/1989
+    if test "$docker_root_dir" ; then
+        LogPrint "Docker is running, skipping filesystems mounted below Docker Root Dir $docker_root_dir"
+    else
+        LogPrintError "Cannot determine Docker Root Dir - things may go wrong - check $DISKLAYOUT_FILE"
+    fi
+fi
 
-# Begin writing output to DISKLAYOUT_FILE:
+# Begin of the subshell that appends its stdout to DISKLAYOUT_FILE:
 (
     echo "# Filesystems (only $supported_filesystems are supported)."
     echo "# Format: fs <device> <mountpoint> <fstype> [uuid=<uuid>] [label=<label>] [<attributes>]"
     # Read the output of the read_filesystems_command:
     while read device mountpoint fstype options junk ; do
+        Log "Processing filesystem '$fstype' on '$device' mounted at '$mountpoint'"
         # Empty device or mountpoint or fstype may may indicate an error. In this case be verbose and inform the user:
         if test -z "$device" -o -z "$mountpoint" -o -z "$fstype" ; then
-            LogPrint "Empty device='$device' or mountpoint='$mountpoint' or fstype='$fstype', skipping saving filesystem layout for it."
+            LogPrintError "Empty device='$device' or mountpoint='$mountpoint' or fstype='$fstype', skipping saving filesystem layout for it."
             continue
         fi
         # FIXME: I (jsmeix@suse.de) have no idea what the reason for the following is.
@@ -88,15 +104,18 @@ service docker status >/dev/null 2>&1 && docker_is_running="yes"
             Log "$device is CD/DVD type device [fstype=$fstype], skipping."
             continue
         fi
-        # docker specific exclude part
+        # Docker specific exclude part:
         if is_true $docker_is_running ; then
-            # docker daemon/service is running
-            docker_root_dir=$( docker info 2>/dev/null | grep 'Docker Root Dir' | awk '{print $4}' )
-            # If $docker_root_dir is in the beginning of the $mountpoint string then FS is under docker control
-            # and we better exclude from saving the layout,
-            # see https://github.com/rear/rear/issues/1749
-            Log "$device is mounted below $docker_root_dir (mount point $mountpoint is under docker control), skipping."
-            echo "$mountpoint" | grep -q "^$docker_root_dir" && continue
+            # If docker_root_dir is the beginning of the mountpoint string then the filesystem is under Docker control
+            # and we better exclude it from saving the layout, see https://github.com/rear/rear/issues/1749
+            # but ensure docker_root_dir is not empty (otherwise any mountpoint string matches "^" which
+            # would skip all mountpoints), see https://github.com/rear/rear/issues/1989#issuecomment-456054278
+            if test "$docker_root_dir" ; then
+                if echo "$mountpoint" | grep -q "^$docker_root_dir" ; then
+                    Log "Filesystem $fstype on $device mounted at $mountpoint is below Docker Root Dir $docker_root_dir, skipping."
+                    continue
+                fi
+            fi
         fi
         # Replace a symbolic link /dev/disk/by-uuid/a1b2c3 -> ../../sdXn
         # by the fully canonicalized target of the link e.g. /dev/sdXn
@@ -201,8 +220,9 @@ service docker status >/dev/null 2>&1 && docker_is_running="yes"
         echo
     done < <( eval $read_filesystems_command )
 
-    # Btrfs subvolume layout if a btrfs filesystem exists:
+    # Begin btrfs subvolume layout if a btrfs filesystem exists:
     if test -n "$btrfs_devices_and_mountpoints" ; then
+        btrfs_subvolume_sles_setup_devices=""
         ########################################
         # Btrfs subvolumes (regardless if mounted or not):
         for btrfs_device_and_mountpoint in $btrfs_devices_and_mountpoints ; do
@@ -232,7 +252,7 @@ service docker status >/dev/null 2>&1 && docker_is_running="yes"
             test -z "$btrfs_default_subvolume_path" && btrfs_default_subvolume_path="/"
             # Empty btrfs_default_subvolume_ID may may indicate an error. In this case be verbose and inform the user:
             if test -z "$btrfs_default_subvolume_ID" ; then
-                LogPrint "Empty btrfs_default_subvolume_ID, no btrfs default subvolume stored for $btrfs_device at $btrfs_mountpoint"
+                LogPrintError "Empty btrfs_default_subvolume_ID, no btrfs default subvolume stored for $btrfs_device at $btrfs_mountpoint"
             else
                 echo "btrfsdefaultsubvol $btrfs_device $btrfs_mountpoint $btrfs_default_subvolume_ID $btrfs_default_subvolume_path"
             fi
@@ -283,9 +303,22 @@ service docker status >/dev/null 2>&1 && docker_is_running="yes"
                 # cf. similar code in layout/prepare/GNU/Linux/130_include_mount_subvolumes_code.sh
                 SLES12SP1_btrfs_detection_string="@/.snapshots/"
                 if btrfs subvolume get-default $btrfs_mountpoint | grep -q "$SLES12SP1_btrfs_detection_string" ; then
-                    info_message="Doing SLES12-SP1 (and later) btrfs subvolumes setup because the default subvolume path contains '$SLES12SP1_btrfs_detection_string'"
+                    info_message="SLES12-SP1 (and later) btrfs subvolumes setup needed for $btrfs_device (default subvolume path contains '$SLES12SP1_btrfs_detection_string')"
+                    if is_false "$BTRFS_SUBVOLUME_SLES_SETUP" ; then
+                        Error "BTRFS_SUBVOLUME_SLES_SETUP is false but $info_message"
+                    fi
                     LogPrint $info_message
                     echo "# $info_message"
+                    # Append all btrfs filesystem device nodes with SLES12-SP1 (and later) btrfs subvolumes setup needed
+                    # to the BTRFS_SUBVOLUME_SLES_SETUP array (unless such a device is already in that array)
+                    # to enforce during "rear recover" btrfs_subvolumes_setup_SLES() is called to setup that btrfs filesystem
+                    # regardless what there already is in BTRFS_SUBVOLUME_SLES_SETUP - e.g. if BTRFS_SUBVOLUME_SLES_SETUP=( 'false' )
+                    # but /dev/sda3 is a btrfs filesystem device node with SLES12-SP1 (and later) btrfs subvolumes setup needed
+                    # it will become BTRFS_SUBVOLUME_SLES_SETUP=( 'false' '/dev/sda3' ) which avoids btrfs_subvolumes_setup_SLES()
+                    # for all devices except '/dev/sda3' where btrfs_subvolumes_setup_SLES() is called to setup that btrfs filesystem
+                    # cf. https://github.com/rear/rear/pull/2080#discussion_r265046317 and see the code in the
+                    # usr/share/rear/layout/prepare/GNU/Linux/133_include_mount_filesystem_code.sh script:
+                    IsInArray "$btrfs_device" "${BTRFS_SUBVOLUME_SLES_SETUP[@]}" || btrfs_subvolume_sles_setup_devices="$btrfs_subvolume_sles_setup_devices $btrfs_device"
                     # SLES 12 SP1 (or later) normal subvolumes that belong to snapper are excluded from being recreated:
                     # Snapper's base subvolume '/@/.snapshots' is excluded because during "rear recover"
                     # that one will be created by "snapper/installation-helper --step 1" which fails if it already exists
@@ -450,8 +483,58 @@ service docker status >/dev/null 2>&1 && docker_is_running="yes"
         else
             echo "# Attributes cannot be determined because no executable 'lsattr' and/or 'findmnt' command(s) found that supports 'FSROOT'."
         fi
+
+        # This needs to be done inside the subshell (that appends its stdout to DISKLAYOUT_FILE)
+        # because outside the subshell the inside the subshell set variables would be lost:
+        if test "$btrfs_subvolume_sles_setup_devices" ; then
+            # Save the updated BTRFS_SUBVOLUME_SLES_SETUP array variable that is needed in recover mode into the rescue.conf file:
+            cat - <<EOF >> "$ROOTFS_DIR/etc/rear/rescue.conf"
+# During "rear mkbackup/mkrescue" via usr/share/rear/layout/save/GNU/Linux/230_filesystem_layout.sh
+# all btrfs filesystem device nodes with SLES12-SP1 (and later) btrfs subvolumes setup needed
+# were appended to the BTRFS_SUBVOLUME_SLES_SETUP array (unless such a device was already in that array)
+# to enforce during "rear recover" btrfs_subvolumes_setup_SLES() gets called to setup that btrfs filesystem
+# (see the usr/share/rear/layout/prepare/GNU/Linux/133_include_mount_filesystem_code.sh script):
+BTRFS_SUBVOLUME_SLES_SETUP=( ${BTRFS_SUBVOLUME_SLES_SETUP[@]} $btrfs_subvolume_sles_setup_devices )
+
+EOF
+            # The rescue.conf file is sourced last by usr/sbin/rear i.e. after site.conf and local.conf
+            # so that the settings in rescue.conf have highest priority.
+            LogPrint "Added $btrfs_subvolume_sles_setup_devices to BTRFS_SUBVOLUME_SLES_SETUP in $ROOTFS_DIR/etc/rear/rescue.conf"
+        fi
+
+
+    # End btrfs subvolume layout if a btrfs filesystem exists:
     fi
 
 ) >> $DISKLAYOUT_FILE
-# End writing output to DISKLAYOUT_FILE.
+# End of the subshell that appends its stdout to DISKLAYOUT_FILE.
+
+# mkfs is required in the recovery system if disklayout.conf contains at least one 'fs' entry
+# see the create_fs function in layout/prepare/GNU/Linux/130_include_filesystem_code.sh
+# what program calls are written to diskrestore.sh
+# cf. https://github.com/rear/rear/issues/1963
+grep -q '^fs ' $DISKLAYOUT_FILE && REQUIRED_PROGS=( "${REQUIRED_PROGS[@]}" mkfs )
+# Other filesystem creating tools are required in the recovery system
+# depending on which filesystem types entries exist in disklayout.conf
+# (see above supported_filesystems="ext2,ext3,ext4,vfat,xfs,reiserfs,btrfs"):
+required_mkfs_tools=""
+for filesystem_type in $( echo $supported_filesystems | tr ',' ' ' ) ; do
+    grep -q "^fs .* $filesystem_type " $DISKLAYOUT_FILE && required_mkfs_tools="$required_mkfs_tools mkfs.$filesystem_type"
+done
+# Remove duplicates because in disklayout.conf there can be many entries with same filesystem type:
+required_mkfs_tools="$( echo $required_mkfs_tools | tr ' ' '\n' | sort -u | tr '\n' ' ' )"
+REQUIRED_PROGS=( "${REQUIRED_PROGS[@]}" $required_mkfs_tools )
+# mke2fs is also required in the recovery system if any 'mkfs.ext*' filesystem creating tool is required
+# and tune2fs or tune4fs is used to set tunable filesystem parameters on ext2/ext3/ext4
+# see above how $tunefs is set to tune2fs or tune4fs
+echo $required_mkfs_tools | grep -q 'mkfs.ext' && REQUIRED_PROGS=( "${REQUIRED_PROGS[@]}" mke2fs $tunefs )
+# xfs_admin is also required in the recovery system if 'mkfs.xfs' is required:
+echo $required_mkfs_tools | grep -q 'mkfs.xfs' && REQUIRED_PROGS=( "${REQUIRED_PROGS[@]}" xfs_admin )
+# reiserfstune is also required in the recovery system if 'mkfs.reiserfs' is required:
+echo $required_mkfs_tools | grep -q 'mkfs.reiserfs' && REQUIRED_PROGS=( "${REQUIRED_PROGS[@]}" reiserfstune )
+# btrfs is also required in the recovery system if 'mkfs.btrfs' is required
+# cf. what prepare/GNU/Linux/130_include_mount_subvolumes_code.sh writes to diskrestore.sh
+echo $required_mkfs_tools | grep -q 'mkfs.btrfs' && REQUIRED_PROGS=( "${REQUIRED_PROGS[@]}" btrfs )
+
 Log "End saving filesystem layout"
+
