@@ -47,3 +47,10 @@ Log "Saving Swap information."
         echo "swap $filename uuid=$uuid label=$label"
     done < /proc/swaps
 ) >> $DISKLAYOUT_FILE
+
+# mkswap is required in the recovery system if disklayout.conf contains at least one 'swap' entry
+# see the create_swap function in layout/prepare/GNU/Linux/140_include_swap_code.sh
+# what program calls are written to diskrestore.sh
+# cf. https://github.com/rear/rear/issues/1963
+grep -q '^swap ' $DISKLAYOUT_FILE && REQUIRED_PROGS=( "${REQUIRED_PROGS[@]}" mkswap ) || true
+
