@@ -108,49 +108,38 @@ function descendants_pids () {
     kill -0 $parent_pid 2>/dev/null && echo $parent_pid || return 0
 }
 
-# Do all exit tasks:
-function DoExitTasks () {
-    # First of all restore the ReaR default bash flags and options (see usr/sbin/rear)
-    # because otherwise in case of a bash error exit when e.g. "set -e -u -o pipefail" was set
-    # all the exit tasks related code would also run with "set -e -u -o pipefail" still set
-    # which may abort exit tasks related code anywhere with a "sudden death" bash error exit
-    # where in particular no longer the EXIT_FAIL_MESSAGE (cf. below) would be shown
-    # so that for the user ReaR would "just somehow silently abort" in this case
-    # cf. https://github.com/rear/rear/issues/1747#issuecomment-371055121
-    # and https://github.com/rear/rear/issues/700#issuecomment-327755633
-    apply_bash_flags_and_options_commands "$DEFAULT_BASH_FLAGS_AND_OPTIONS_COMMANDS"
-    # Apply debugscript mode also for the exit tasks:
-    test "$DEBUGSCRIPTS" && set -$DEBUGSCRIPTS_ARGUMENT
-    LogPrint "Exiting $PROGRAM $WORKFLOW (PID $MASTER_PID) and its descendant processes"
-    # First of all wait one second to let descendant processes terminate on their own
-    # e.g. after Ctrl+C by the user descendant processes should terminate on their own
-    # at least the "foreground processes" (with the current terminal process group ID)
-    # but "background processes" would not terminate on their own after Ctrl+C
-    # cf. https://github.com/rear/rear/issues/1712
-    sleep 1
-    # Show descendant processes PIDs with their commands in the log
-    # so that later the plain PIDs in the log get more comprehensible.
-    # What works sufficiently on all systems is "pstree -Aplau $MASTER_PID"
+# Show descendant processes PIDs with their commands in the log
+# so that later the plain PIDs in the log get more comprehensible
+# (e.g. when terminate_descendants_pids is called afterwards):
+function log_descendants_pids () {
+    # What works sufficiently on all systems is "pstree -Aplau MASTER_PID"
     # but the pstree command is not available by default in the ReaR recovery system
     # (cf. https://github.com/rear/rear/issues/1755) so that the ps command is used as fallback.
-    # Because "ps f -g $MASTER_PID -o pid,args" only works on older systems like SLES10 and SLES11
+    # Because "ps f -g MASTER_PID -o pid,args" only works on older systems like SLES10 and SLES11
     # (cf. the above comment for the descendants_pids function)
-    # a last resort fallback "ps --ppid $MASTER_PID -o pid,args" is used for newer systems like SLES12
-    # (at least on SLES12 "ps f -g $MASTER_PID -o pid,args" results non-zero exit code when nothing is shown):
+    # a last resort fallback "ps --ppid MASTER_PID -o pid,args" is used for newer systems like SLES12
+    # (at least on SLES12 "ps f -g MASTER_PID -o pid,args" results non-zero exit code when nothing is shown):
     Log "$( pstree -Aplau $MASTER_PID || ps f -g $MASTER_PID -o pid,args || ps --ppid $MASTER_PID -o pid,args )"
+}
+
+# Terminate all still running descendant processes of MASTER_PID but do not terminate the MASTER_PID process itself.
+# First terminate great-grandchildren processes, then grandchildren processes, then children processes.
+# This termination functionality is used in the DoExitTasks() function.
+function terminate_descendants_from_grandchildren_to_children () {
     # Some descendant processes commands could be much too long (e.g. a 'tar ...' command)
     # to be usefully shown completely in the below LogPrint information (could be many lines)
     # so that the descendant process command output is truncated after at most remaining_columns:
     local remaining_columns=$(( COLUMNS - 40 ))
     test $remaining_columns -ge 40 || remaining_columns=40
-    # Terminate all still running descendant processes of $MASTER_PID
-    # but do not termiate the MASTER_PID process itself because
+    # Terminate all still running descendant processes of MASTER_PID
+    # but do not terminate the MASTER_PID process itself because
     # the MASTER_PID process must run the exit tasks below:
     local descendant_pid=""
     local not_yet_terminated_pids=""
+    # Send SIGTERM to all still running descendant processes of MASTER_PID:
     for descendant_pid in $( descendants_pids $MASTER_PID ) ; do
         # The descendant_pids() function outputs at least MASTER_PID
-        # plus the PID of the subshell of the $( descendants_pids $MASTER_PID )
+        # plus the PID of the subshell of the $( descendants_pids MASTER_PID )
         # so that it is tested that a descendant_pid is not MASTER_PID
         # and that a descendant_pid is still running before SIGTERM is sent:
         test $MASTER_PID -eq $descendant_pid && continue
@@ -169,25 +158,154 @@ function DoExitTasks () {
         fi
     done
     # No need to kill a descendant processes if all were already terminated:
-    if test "$not_yet_terminated_pids" ; then
-        # Kill all not yet terminated descendant processes:
-        for descendant_pid in $not_yet_terminated_pids ; do
-            if kill -0 $descendant_pid ; then
-                LogPrint "Killing descendant process $descendant_pid $( ps -p $descendant_pid -o args= | cut -b-$remaining_columns )"
-                kill -SIGKILL $descendant_pid 1>&2
-                # For each killed descendant process wait one second to let it die to be on the safe side
-                # that e.g. grandchildren were actually removed by the kernel before children get SIGKILL sent
-                # i.e. every child process is already gone before its parent process may get SIGKILL so that
-                # the parent (that may wait for its child) has a better chance to still cleanly terminate:
-                sleep 1
-                kill -0 $descendant_pid && LogPrint "Killed descendant process $descendant_pid still there"
-            else
-                # Show a counterpart message to the above 'not yet terminated' message
-                # e.g. after a child process was killed its parent may have terminated on its own:
-                LogPrint "Descendant process $descendant_pid terminated"
-            fi
-        done
+    test "$not_yet_terminated_pids" || return 0
+    # Kill all not yet terminated descendant processes:
+    for descendant_pid in $not_yet_terminated_pids ; do
+        if kill -0 $descendant_pid ; then
+            LogPrint "Killing descendant process $descendant_pid $( ps -p $descendant_pid -o args= | cut -b-$remaining_columns )"
+            kill -SIGKILL $descendant_pid 1>&2
+            # For each killed descendant process wait one second to let it die to be on the safe side
+            # that e.g. grandchildren were actually removed by the kernel before children get SIGKILL sent
+            # i.e. every child process is already gone before its parent process may get SIGKILL so that
+            # the parent (that may wait for its child) has a better chance to still cleanly terminate:
+            sleep 1
+            kill -0 $descendant_pid && LogPrint "Killed descendant process $descendant_pid still there"
+        else
+            # Show a counterpart message to the above 'not yet terminated' message
+            # e.g. after a child process was killed its parent may have terminated on its own:
+            LogPrint "Descendant process $descendant_pid terminated"
+        fi
+    done
+}
+
+# Terminate all still running descendant processes of MASTER_PID but do not terminate the MASTER_PID process itself.
+# First children processes, then grandchildren processes, then great-grandchildren processes.
+# This termination functionality is used in the Error() function.
+# The following code is basically the same as in terminate_descendants_from_grandchildren_to_children (see there for explanatory comments)
+# except small but crucial differences here which is the reason why that kind of code exists two times.
+function terminate_descendants_from_children_to_grandchildren () {
+    # Some descendant processes commands could be much too long (e.g. a 'tar ...' command):
+    local remaining_columns=$(( COLUMNS - 40 ))
+    test $remaining_columns -ge 40 || remaining_columns=40
+    # Terminate all still running descendant processes of MASTER_PID
+    # but do not terminate the MASTER_PID process itself because
+    # the MASTER_PID process must run the exit tasks below
+    # and do not terminate the current process that runs this code here.
+    local current_pid=""
+    local descendant_pid=""
+    local not_yet_terminated_pids=""
+    local descendant_pids_from_children_to_parent="$( descendants_pids $MASTER_PID )"
+    # Reverse the ordering of the PIDs to get them from parent to children:
+    local descendant_pids_from_parent_to_children=""
+    for descendant_pid in $descendant_pids_from_children_to_parent ; do
+        descendant_pids_from_parent_to_children="$descendant_pid $descendant_pids_from_parent_to_children"
+    done
+    # Send SIGTERM to all still running descendant processes of MASTER_PID
+    # except the current process that runs this code here which is usually MASTER_PID
+    # but this code here could be also run within a (possibly deeply nested) subshell:
+    if test "$BASHPID" ; then
+        current_pid=$BASHPID
+    else
+        # When there is no BASHPID we need to determine our current PID indirectly.
+        # Things like https://stackoverflow.com/questions/20725925/get-pid-of-current-subshell
+        # to get the current PID by calling a subshell like "( : ; bash -c 'echo $PPID' )"
+        # do not work when the current PID is already a (possibly deeply nested) subshell.
+        # Interestingly on command line "mypid=$( bash -c 'echo $PPID' )"
+        # works even in a nested subshell but it does no longer work when it is used in a sourced script.
+        # One way that works is that our current PID is the parent PID of a command that is called directly here
+        # (without any indirection via another subshell like "current_pid=$( whatever_command )" or when using a pipe)
+        # like "tmpfile=$( mktemp ) ; cat /proc/self/stat >$tmpfile ; current_pid=$( cut -d ' ' -f4 $tmpfile ) ; rm $tmpfile"
+        # but the simplest way is using the bash builtin 'read' to get our current PID directly from /proc/self/stat
+        # (our current PID is the first field in /proc/self/stat and our parent PID is the fourth field):
+        read current_pid junk </proc/self/stat
     fi
+    for descendant_pid in $descendant_pids_from_parent_to_children ; do
+        # Test that a descendant_pid is not MASTER_PID or the current process that runs this code here
+        # and that a descendant_pid is still running before SIGTERM is sent:
+        test $MASTER_PID -eq $descendant_pid && continue
+        test $current_pid -eq $descendant_pid && continue
+        kill -0 $descendant_pid || continue
+        LogPrint "Terminating child process $descendant_pid $( ps -p $descendant_pid -o args= | cut -b-$remaining_columns )"
+        kill -SIGTERM $descendant_pid 1>&2
+    done
+    # In contrast to the terminate_descendants_from_grandchildren_to_children function above
+    # we do not wait here one second for each processes when it gets SIGTERM above
+    # because we send SIGTERM first to children then to grandchildren
+    # so that it does not make sense to give a grandchild one second
+    # to let it cleanly terminate before its paretnt child gets SIGTERM.
+    # Wait one second to let the above processes that got SIGTERM actually terminate
+    # before determining which did not yet terminate and should get a SIGKILL:
+    sleep 1
+    # Determine which of the above processes that got SIGTERM did not yet terminate
+    # except MASTER_PID and the current process that runs this code here:
+    for descendant_pid in $descendant_pids_from_parent_to_children ; do
+        test $MASTER_PID -eq $descendant_pid && continue
+        test $current_pid -eq $descendant_pid && continue
+        if kill -0 $descendant_pid ; then
+            # Keep the current ordering also in not_yet_terminated_pids
+            # i.e. children before grandchildren:
+            not_yet_terminated_pids="$not_yet_terminated_pids $descendant_pid"
+            LogPrint "Child process $descendant_pid not yet terminated"
+        fi
+    done
+    # No need to kill a descendant processes if all were already terminated:
+    test "$not_yet_terminated_pids" || return 0
+    # Kill all not yet terminated descendant processes that already got SIGTERM above:
+    for descendant_pid in $not_yet_terminated_pids ; do
+        if kill -0 $descendant_pid ; then
+            LogPrint "Killing child process $descendant_pid $( ps -p $descendant_pid -o args= | cut -b-$remaining_columns )"
+            kill -SIGKILL $descendant_pid 1>&2
+        else
+            # Show a counterpart message to the above 'not yet terminated' message:
+            LogPrint "Child process $descendant_pid terminated"
+        fi
+    done
+    # In contrast to the terminate_descendants_from_grandchildren_to_children function above
+    # we do not wait here one second for each processes when it gets SIGKILL above
+    # with the same reasoning behind as above where SIGTERM was sent.
+    # Wait one second the let the killed descendant processes actually die:
+    sleep 1
+    # Show which killed descendant processes are still there:
+    for descendant_pid in $not_yet_terminated_pids ; do
+        kill -0 $descendant_pid && LogPrint "Killed child process $descendant_pid still there"
+    done
+}
+
+# Do all exit tasks:
+function DoExitTasks () {
+    # First of all restore the ReaR default bash flags and options (see usr/sbin/rear)
+    # because otherwise in case of a bash error exit when e.g. "set -e -u -o pipefail" was set
+    # all the exit tasks related code would also run with "set -e -u -o pipefail" still set
+    # which may abort exit tasks related code anywhere with a "sudden death" bash error exit
+    # where in particular no longer the EXIT_FAIL_MESSAGE (cf. below) would be shown
+    # so that for the user ReaR would "just somehow silently abort" in this case
+    # cf. https://github.com/rear/rear/issues/1747#issuecomment-371055121
+    # and https://github.com/rear/rear/issues/700#issuecomment-327755633
+    # To avoid useless 'set -x' debug output for the apply_bash_flags_and_options_commands call
+    # run it in the current shell environment where stderr is redirected to /dev/null before:
+    { apply_bash_flags_and_options_commands "$DEFAULT_BASH_FLAGS_AND_OPTIONS_COMMANDS" ; } 2>/dev/null
+    # Apply debugscript mode also for the exit tasks:
+    test "$DEBUGSCRIPTS" && set -$DEBUGSCRIPTS_ARGUMENT
+    LogPrint "Exiting $PROGRAM $WORKFLOW (PID $MASTER_PID) and its descendant processes ..."
+    # Wait some time to let descendant processes terminate on their own
+    # e.g. after Ctrl+C by the user descendant processes should terminate on their own
+    # at least the "foreground processes" (with the current terminal process group ID)
+    # but "background processes" would not terminate on their own after Ctrl+C
+    # cf. https://github.com/rear/rear/issues/1712
+    # and also the Error function terminates descendant processes on its own via
+    # terminate_descendants_from_children_to_grandchildren that sleeps two times one second
+    # so that we wait here three seconds to be on the safe side that a possibly running
+    # terminate_descendants_from_children_to_grandchildren has done its job and finished
+    # to avoid that two functions run in parallel that terminate descendant processes:
+    sleep 3
+    # Show descendant processes PIDs with their commands in the log
+    # so that the plain PIDs in the log get more comprehensible
+    # when terminate_descendants_from_grandchildren_to_children is called afterwards:
+    log_descendants_pids
+    # Terminate all still running descendant processes of MASTER_PID
+    # but do not terminate the MASTER_PID process itself because
+    # the MASTER_PID process must run the exit tasks below:
+    terminate_descendants_from_grandchildren_to_children
     # Finally run the exit tasks:
     LogPrint "Running exit tasks"
     local exit_task=""
@@ -207,8 +325,8 @@ builtin trap "DoExitTasks" EXIT
 # (which is usually the keyboard and display of the user who launched 'rear')
 # the original STDIN STDOUT and STDERR file descriptors are saved as fd6 fd7 and fd8
 # so that ReaR functions for actually intended user messages can use fd7 and fd8
-# to show messages to the user regardless whereto STDOUT and STDERR are redirected
-# and fd6 to get input from the user regardless whereto STDIN is redirected.
+# to show messages to the user regardless where to STDOUT and STDERR are redirected
+# and fd6 to get input from the user regardless where to STDIN is redirected.
 # Duplicate STDIN to fd6 to be used by 'read' in the UserInput function
 # cf. http://tldp.org/LDP/abs/html/x17974.html
 exec 6<&0
@@ -235,7 +353,7 @@ QuietAddExitTask "(( EXIT_FAIL_MESSAGE )) && echo '${MESSAGE_PREFIX}$PROGRAM $WO
 
 # USR1 is used to abort on errors.
 # It is not using PrintError but does direct output to the original STDERR.
-# Set EXIT_FAIL_MESSAGE to 0 to aviod an additional failed message via the QuietAddExitTask above:
+# Set EXIT_FAIL_MESSAGE to 0 to avoid an additional failed message via the QuietAddExitTask above:
 builtin trap "EXIT_FAIL_MESSAGE=0 ; echo '${MESSAGE_PREFIX}Aborting due to an error, check $RUNTIME_LOGFILE for details' 1>&8 ; kill $MASTER_PID" USR1
 
 # Make sure nobody else can use trap:
@@ -246,7 +364,12 @@ function trap () {
 # For actually intended user messages output to the original STDOUT
 # but only when the user launched 'rear -v' in verbose mode:
 function Print () {
-    test "$VERBOSE" && echo -e "${MESSAGE_PREFIX}$*" 1>&7 || true
+    # It is crucial to append to /dev/$DISPENSABLE_OUTPUT_DEV when $DISPENSABLE_OUTPUT_DEV is not 'null'.
+    # In debugscripts mode $DISPENSABLE_OUTPUT_DEV is 'stderr' (see usr/sbin/rear)
+    # and /dev/stderr is fd2 which is redirected to append to RUNTIME_LOGFILE (see usr/sbin/rear)
+    # so that 2>/dev/stderr would truncate RUNTIME_LOGFILE to zero size (see 'REDIRECTION' in "man bash")
+    # but 2>>/dev/stderr does not change things so that fd2 output is still appended to RUNTIME_LOGFILE:
+    { test "$VERBOSE" && echo "${MESSAGE_PREFIX}$*" 1>&7 || true ; } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
 }
 
 # For normal output messages that are intended for user dialogs.
@@ -256,13 +379,13 @@ function Print () {
 # but output to the original STDOUT without a MESSAGE_PREFIX because
 # MESSAGE_PREFIX is not helpful in normal user dialog output messages:
 function UserOutput () {
-    echo -e "$*" 1>&7 || true
+    { echo "$*" 1>&7 || true ; } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
 }
 
 # For actually intended user error messages output to the original STDERR
 # regardless whether or not the user launched 'rear' in verbose mode:
 function PrintError () {
-    echo -e "${MESSAGE_PREFIX}$*" 1>&8 || true
+    { echo "${MESSAGE_PREFIX}$*" 1>&8 || true ; } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
 }
 
 # For messages that should only appear in the log file output to the current STDERR
@@ -270,12 +393,15 @@ function PrintError () {
 function Log () {
     # Have a timestamp with nanoseconds precision in any case
     # so that any subsequent Log() calls get logged with precise timestamps:
-    local timestamp=$( date +"%Y-%m-%d %H:%M:%S.%N " )
-    if test $# -gt 0 ; then
-        echo "${MESSAGE_PREFIX}${timestamp}$*" || true
-    else
-        echo "${MESSAGE_PREFIX}${timestamp}$( cat )" || true
-    fi 1>&2
+    { local timestamp=$( date +"%Y-%m-%d %H:%M:%S.%N " )
+      local log_message=""
+      if test $# -gt 0 ; then
+          log_message="${MESSAGE_PREFIX}${timestamp}$*"
+      else
+          log_message="${MESSAGE_PREFIX}${timestamp}$( cat )"
+      fi
+    } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
+    echo "$log_message" 1>&2 || true
 }
 
 # For messages that should only appear in the log file when the user launched 'rear -d' in debug mode:
@@ -324,10 +450,11 @@ function LogToSyslog () {
 # see https://github.com/rear/rear/issues/729
 function has_binary () {
     for bin in $@ ; do
-        # Suppress success output via stdout (but keep failure output via stderr):
-        if type $bin 1>/dev/null ; then
-            return 0
-        fi
+        # Suppress success output via stdout which is crucial when has_binary is called
+        # in other functions that provide their intended function results via stdout
+        # to not pollute intended function results with intermixed has_binary stdout
+        # (e.g. the RequiredSharedObjects function) but keep failure output via stderr:
+        type $bin 1>/dev/null && return 0
     done
     return 1
 }
@@ -341,38 +468,7 @@ function get_path () {
     type -P $1
 }
 
-# Error exit:
-function Error () {
-    LogPrintError "ERROR: $*"
-    LogToSyslog "ERROR: $*"
-    # TODO: I <jsmeix@suse.de> wonder if the "has_binary caller" test is still needed nowadays
-    # because for me on SLE10 with bash-3.1-24 up to SLE12 with bash-4.2 'caller' is a shell builtin:
-    if has_binary caller ; then
-        # Print stack strace in reverse order to the current STDERR which is (usually) the log file:
-        (   echo "==== ${MESSAGE_PREFIX}Stack trace ===="
-            local c=0;
-            while caller $((c++)) ; do
-                # nothing to do
-                :
-            done | awk ' { l[NR]=$3":"$1" "$2 }
-                         END { for (i=NR; i>0;) print "Trace "NR-i": "l[i--] }
-                       '
-            echo "${MESSAGE_PREFIX}Message: $*"
-            echo "== ${MESSAGE_PREFIX}End stack trace =="
-        ) 1>&2
-    fi
-    # Make sure Error exits the master process, even if called from child processes:
-    kill -USR1 $MASTER_PID
-}
-
-# If return code is non-zero, bail out:
-function StopIfError () {
-    if (( $? != 0 )) ; then
-        Error "$@"
-    fi
-}
-
-# Output the source file of the actual caller script:
+# Output the source file of the actual caller script and its line number:
 function CallerSource () {
     # Get the source file of actual caller script.
     # Usually this is ${BASH_SOURCE[1]} but CallerSource is also called
@@ -413,9 +509,175 @@ function CallerSource () {
     echo "Relax-and-Recover"
 }
 
+# Error exit:
+function Error () {
+    # Get the last sourced script out of the log file:
+    # Using the CallerSource function is not sufficient here because CallerSource results
+    # the file where this Error function is called which can also be a lib/*-functions.sh
+    # but showing *-functions.sh would not be as helpful for the user as the last actual script.
+    # Each sourced script gets logged as 'timestamp Including sub-path/to/script_file_name.sh' and
+    # valid script files names are of the form NNN_script_name.sh (i.e. with leading 3-digit number)
+    # but also the outdated scripts with leading 2-digit number get sourced
+    # see the SourceStage function in lib/framework-functions.sh
+    # so that we grep for script files names with two or more leading numbers:
+    { local last_sourced_script_log_entry=( $( grep -o ' Including .*/[0-9][0-9].*\.sh' $RUNTIME_LOGFILE | tail -n 1 ) )
+      # The last_sourced_script_log_entry contains: Including sub-path/to/script_file_name.sh
+      local last_sourced_script_sub_path="${last_sourced_script_log_entry[1]}"
+      local last_sourced_script_filename="$( basename $last_sourced_script_sub_path )"
+      # When it errors out in sbin/rear last_sourced_script_filename is empty which would result bad looking output
+      # cf. https://github.com/rear/rear/issues/1965#issuecomment-439437868
+      test "$last_sourced_script_filename" || last_sourced_script_filename="$SCRIPT_FILE"
+    } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
+    # Do not log the error message right now but after the currenly last log messages were shown:
+    PrintError "ERROR: $*"
+    # Show some additional hopefully meaningful output on the user's terminal
+    # (no need to log that again here because it is already in the log file)
+    # in particular the normal stdout and stderr messages of the last called programs
+    # to make the root cause more obvious to the user without the need to analyze the log file
+    # cf. https://github.com/rear/rear/issues/1875#issuecomment-407039065
+    PrintError "Some latest log messages since the last called script $last_sourced_script_filename:"
+    # Extract lines starting when the last script was sourced (logged as 'Including sub-path/to/script.sh')
+    # but do not use last_sourced_script_sub_path because it contains '/' characters that let sed fail with
+    #   sed: -e expression #1, char ...: extra characters after command
+    # because the '/' characters would need to be escaped in the sed expression so that
+    # we simply use last_sourced_script_filename in the sed expression.
+    # Extract at most up to a line that is usually logged as '++ Error ...' or '++ BugError ...'
+    # (but do not stop at lines that are logged like '++ StopIfError ...' or '++ PrintError ...')
+    # if such a '+ Error' or '+ BugError' line exists, otherwise sed proceeds to the end
+    # (the sed pattern '[Bug]*Error' is fuzzy because it would also match things like 'uuggError').
+    # The reason to stop at a line that contains '+ [Bug]*Error ' is that in debugscripts mode '-D'
+    # a BugError or Error function call with a multi line error message (e.g. BugError does that)
+    # results 'set -x' debug output of that function call in the log file that looks like:
+    #   ++ [Bug]Error 'first error message line
+    #   second error message line
+    #   third error message line
+    #   ...
+    #   last error message line'
+    # Because of the newlines in the error message subsequent lines appear without a leading '+' character
+    # so that those debug output lines are indistinguishable from normal stdout/stderr output of programs,
+    # cf. https://github.com/rear/rear/pull/1877
+    # Thereafter ('+ [Bug]*Error ' lines were needed before) skip 'set -x' lines (lines that start with a '+' character).
+    # Show at most the last 8 lines because too much before the actual error may cause more confusion than help.
+    # Add two spaces indentation for better readability what those extracted log file lines are.
+    # Some messages could be too long to be usefully shown on the user's terminal so that they are truncated after 200 bytes:
+    { local last_sourced_script_log_messages="$( sed -n -e "/Including .*$last_sourced_script_filename/,/+ [Bug]*Error /p" $RUNTIME_LOGFILE | grep -v '^+' | tail -n 8 | sed -e 's/^/  /' | cut -b-200 )" ; } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
+    PrintError "$last_sourced_script_log_messages"
+    Log "ERROR: $*"
+    LogToSyslog "ERROR: $*"
+    # Print stack strace in reverse order to the current STDERR which is (usually) the log file:
+    ( echo "===== ${MESSAGE_PREFIX}Stack trace ====="
+      local c=0;
+      while caller $((c++)) ; do
+          :
+      done | awk ' { l[NR]=$3":"$1" "$2 }
+                   END { for (i=NR; i>0;) print "Trace "NR-i": "l[i--] }
+                 '
+      echo "=== ${MESSAGE_PREFIX}End stack trace ==="
+    ) 1>&2
+    # Make sure Error exits the master process, even if called from child processes.
+    # We must send USR1 to MASTER_PID before we terminate all still running descendant processes of MASTER_PID below
+    # because when the Error function is called from a subshell we are one of those still running descendant processes:
+    kill -USR1 $MASTER_PID
+    # That USR1 has a trap (see above) that does 'kill MASTER_PID' whicht triggers another trap on EXIT that calls DoExitTasks().
+    # When the Error function is called from within a subshell (cf. layout/save/GNU/Linux/230_filesystem_layout.sh) like 
+    #   ( echo "additional content for file" || Error "failed to append content to file" ) >> file
+    # the Error function does not let MASTER_PID exit because the parent shell waits until its subshell has finished
+    # so that the USR1 that was sent above to MASTER_PID will be processed only after the subshell has finished
+    # cf. https://github.com/rear/rear/issues/2089#issuecomment-474260332 that reads (excerpts)
+    #    A nice clean reproducer on plain command line (needs a recent bash that supports BASHPID):
+    #       # export MASTERPID=$BASHPID
+    #       # trap "echo $MASTERPID got USR1" USR1
+    #       # ( echo begin subshell $BASHPID parent $MASTERPID
+    #           pstree -Aplau $MASTERPID
+    #           kill -USR1 $MASTERPID
+    #           echo sent USR1 to $MASTERPID in subshell
+    #           echo other stuff in subshell
+    #           echo subshell done )
+    #    Running that reproducer results:
+    #       begin subshell 26109 parent 26108
+    #       bash,26108
+    #         `-bash,26109
+    #             `-pstree,26110 -Aplau 26108
+    #       sent USR1 to 26108 in subshell
+    #       other stuff in subshell
+    #       subshell done
+    #       26108 got USR1
+    #   It shows that the parent waits until its subshell child has finished and then the parent processes the signal.
+    #   This behaviour matches what "man bash" reads for "SIGNALS":
+    #      If bash is waiting for a command to complete
+    #      and receives a signal for which a trap has been set,
+    #      the trap will not be executed until the command completes.
+    # This means when the Error function is called from within a subshell only USR1 is sent to MASTER_PID
+    # and the subshell continues with all its code after the Error function until the subshell finishes.
+    # This would result unintendedly executed code (with all its unexpected messages in the log file) and
+    # also further Error function calls with error messages on the user's terminal from subsequent failures
+    # after the initial error, e.g. see https://github.com/rear/rear/issues/2087#issue-421604286 that shows
+    #   ERROR: Partition number '0' of partition mmcblk0boot0 is not a valid number.
+    #   ERROR: Partition number '' of partition mmcblk0rpmb is not a valid number.
+    #   ERROR: Partition mmcblk0rpmb is numbered ''. More than 128 partitions is not supported.
+    #   Aborting due to an error, check /var/log/rear/rear-testvm02.log for details
+    # where only the first error message should have been shown and a direct abort should have happened.
+    # This is the reason why we have to terminate all still running descendant processes of MASTER_PID
+    # but do not terminate the MASTER_PID process itself because the MASTER_PID process must run
+    # the exit tasks via DoExitTasks via trap on EXIT via trap on USR1 (see above).
+    # How to cleanly error out from within a lower level of nested subshells as in this code:
+    #   ( LogPrint "Begin first subshell"
+    #     ( LogPrint "Begin second subshell"
+    #       Error "First error"
+    #       Error "Second error"
+    #       LogPrint "End second subshell"
+    #     )
+    #     LogPrint "Code in first subshell after second subshell"
+    #     LogPrint "End of first subshell"
+    #   )
+    # It should error out at "First error" and not execute any code after that.
+    # If we terminate the second subshell here (i.e. the one that currently runs this 'Error "First error"' function)
+    # we could avoid that the second subshell unintendedly continues and runs the 'Error "Second error"' function
+    # but its parent (i.e. the first subshell that has waited for its second subshell child to finish)
+    # would then continue and unintendedly execute the "Code in first subshell after second subshell".
+    # Therefore we terminate all still running processes (except MASTER_PID) starting with children to grandchildren
+    # so that we terminate first the first subshell and then the second subshell.
+    # This way when the second subshell gets terminated its parent was already terminated
+    # so that in the end there will be no unintendedly executed code after the "First error".
+    # The following code is basically the same as in DoExitTasks (see there for explanatory comments)
+    # except small but crucial differences here which is the reason why that kind of code exists two times.
+    # First of all restore the ReaR default bash flags and options of MASTER_PID (i.e. of usr/sbin/rear):
+    { apply_bash_flags_and_options_commands "$DEFAULT_BASH_FLAGS_AND_OPTIONS_COMMANDS" ; } 2>/dev/null
+    # Kepp debugscript mode also here if it was used before:
+    test "$DEBUGSCRIPTS" && set -$DEBUGSCRIPTS_ARGUMENT
+    LogPrint "Error exit of $PROGRAM $WORKFLOW (PID $MASTER_PID) and its descendant processes"
+    # Show descendant processes PIDs with their commands in the log
+    # so that the plain PIDs in the log get more comprehensible
+    # when terminate_descendants_from_children_to_grandchildren is called afterwards:
+    log_descendants_pids
+    # Terminate all still running descendant processes of MASTER_PID
+    # but do not terminate the MASTER_PID process itself because
+    # the MASTER_PID process must run the exit tasks via DoExitTasks (see above)
+    # and do not terminate the current process that runs this code here
+    # because the terminate_descendants_from_children_to_grandchildren function
+    # should run to its end because it may have to kill descendant processes:
+    terminate_descendants_from_children_to_grandchildren
+    # Now only the process that runs this code here is left.
+    # If that process is MASTER_PID all is o.k. but if that process is run within a subshell
+    # we must not return here from the Error funtion to its caller because that would let
+    # the subshell continue with all its code after the Error function until the subshell finishes
+    # so that if we are in a subshell here we exit from that subshell here:
+    if test $BASH_SUBSHELL -gt 0 ; then
+        LogPrint "Exiting subshell $BASH_SUBSHELL (where the actual error happened)"
+        test $EXIT_CODE -gt 1 && exit $EXIT_CODE || exit 1
+    fi
+}
+
+# If return code is non-zero, bail out:
+function StopIfError () {
+    if (( $? != 0 )) ; then
+        Error "$@"
+    fi
+}
+
 # Exit if there is a bug in ReaR:
 function BugError () {
-    local caller_source="$( CallerSource )"
+    { local caller_source="$( CallerSource )" ; } 2>>/dev/$DISPENSABLE_OUTPUT_DEV
     Error "
 ====================
 BUG in $caller_source:
@@ -423,7 +685,7 @@ BUG in $caller_source:
 --------------------
 Please report this issue at https://github.com/rear/rear/issues
 and include the relevant parts from $RUNTIME_LOGFILE
-preferably with full debug information via 'rear -d -D $WORKFLOW'
+preferably with full debug information via 'rear -D $WORKFLOW'
 ===================="
 }
 
